#include <iostream>
#include <string>
#include "DataContainer.h"

using namespace std;

DataContainer::DataContainer()
{
}


string DataContainer::getHeader() const
{
    return header;
}

void DataContainer::setHeader(string _header)
{
    header = _header;
}


Data DataContainer::getData(int index) const
{
    return items[index];
}


void DataContainer::addData(Data item)
{
    items.push_back(item);
}


void DataContainer::clear()
{
    items.clear();
}


bool DataContainer::isPeak(int index) const
{
    if (index == 0 || index == items.size() - 1)
        return false;
        
    return items[index - 1].getAdjClose() < items[index].getAdjClose() 
        && items[index + 1].getAdjClose() < items[index].getAdjClose();
}

bool DataContainer::isValley(int index) const
{
    if (index == 0 || index == items.size() - 1)
        return false;
        
    return items[index - 1].getAdjClose() > items[index].getAdjClose() 
        && items[index + 1].getAdjClose() > items[index].getAdjClose();
}

// returns the relative value differences
// at points i and j. (The price at i should be larger than price at j)
//
double DataContainer::valueDifference(int i, int j) const
{
    return (items[i].getAdjClose() - items[j].getAdjClose()) / items[i].getAdjClose();
}

bool DataContainer::doubleTop(int &firstTop, int &confirmation) const
{
    const double minDiff = 0.05; // 5% difference between peak and valley

    for (int i = 0; i < items.size(); ++i)
    {
        if (isPeak(i)) // possible location of the first peak
        {
            for (int j = i + 1; j < items.size(); ++j)
            {
                if (isValley(j) && valueDifference(i, j) > minDiff)  // possible location of the valley
                {
                    for (int k = j + 1; k < items.size(); ++k)
                    {
                        if (isPeak(k) && valueDifference(k, j) > minDiff) // possible location of the second peak
                        {
                            for (int p = k + 1; p < items.size(); p++)
                            {
                                if (items[p].getAdjClose() < items[j].getAdjClose()) // the formation is completed 
                                {
                                    firstTop = i;
                                    confirmation = p;
                                    return true; 
                                }
                            }
                        }
                    }
                }
            }
        }
    }
    
    return false;
}

ostream& operator<<(ostream& os, const DataContainer& d)
{
    os << d.getHeader() << endl;  // write the header
    for (vector<Data>::const_iterator it = d.items.begin() ; it != d.items.end(); ++it)
        os << *it << endl;

    return os;
} 

istream& operator>>(istream& is, DataContainer& d)
{
    string header;
    Data item;

    d.clear();
    
    getline(is, header); // read the header
    d.setHeader(header);
    
    while (is >> item)
        d.addData(item);
        
    return is;
}

